
MOORP_Settings = {} -- savedvar (why did I not call this addon Moorp?)

local slowThrottle = 0.50 -- update speed when not over a unit frame
local midThrottle = 0.10 -- update speed while over a unit that is far away
local fastThrottle = 0.01 -- update speed while unit is relatively near

local settings

local pointer = CreateFrame("Frame",nil,UIParent)
pointer:Hide()
pointer.timer = 0
pointer:SetFrameStrata("FULLSCREEN")

pointer.arrow = CreateFrame("Frame",nil,pointer)
pointer.arrow.texture = pointer.arrow:CreateTexture(nil,"OVERLAY")
pointer.arrow.texture:SetAllPoints(true)
pointer.arrow.texture:SetTexture("Interface\\MiniMap\\MiniMap-QuestArrow")

pointer.yards = pointer.arrow:CreateFontString(nil,"OVERLAY","GameFontHighlight")
pointer.yards:SetPoint("RIGHT",pointer.arrow,"LEFT")
pointer.yardsback = pointer.arrow:CreateTexture(nil,"BACKGROUND")
pointer.yardsback:SetPoint("TOPLEFT",pointer.yards,"TOPLEFT",-3,3)
pointer.yardsback:SetPoint("BOTTOMRIGHT",pointer.yards,"BOTTOMRIGHT",3,-3)
pointer.yardsback:SetTexture(1,1,1)
pointer.yardsback:SetGradientAlpha("VERTICAL",0,0,0,1,.25,.25,.25,.5)

local sqrt2 = sqrt(2) -- stuff to help texcoord mumbo jumbo later
local rads45 = 0.25*PI
local rads135 = 0.75*PI
local rads225 = 1.25*PI
local cos, sin = math.cos, math.sin
local function corner(r) return 0.5+cos(r)/sqrt2, 0.5+sin(r)/sqrt2 end

-- self here is pointer.arrow; update whether to show arrow and direction it points
local function UpdateArrow(self)

	-- see if there's a unit frame under the mouse
	local focus = GetMouseFocus()
	local unit = focus and focus:GetAttribute("unit")
	if not unit then
		self:Hide()
		pointer.timer = slowThrottle
		return
	end

	local playerX, playerY = UnitPosition("player")
	local unitX, unitY = UnitPosition(unit)

	-- UnitInRange() is what dims default raid frames; it doesn't precisely align with UnitDistanceSquared()
	-- so using UnitInRange() for the more-important check if in spell casting range
	local inRange = UnitInRange(unit)

	-- hide arrow if unit does not have coordinates (not in group, pet, etc), or
	-- player and unit are same unit, or unit is near (and ShowInRange not set in options)
	if not playerX or not unitX or UnitIsUnit("player",unit) or (inRange and not MOORP_Settings.ShowInRange) then
		self:Hide()
		pointer.timer = midThrottle
		return
	end

	local distance = UnitDistanceSquared(unit)

	pointer.timer = fastThrottle -- we're going to show an arrow, speed up updates

	local arrow = self.texture

	if inRange then -- less than 40 yards, near: green (distance 1600 is not reliable with UnitInRange--so going with more visible API)
		arrow:SetVertexColor(0,1,0)
	elseif distance<22500 then -- less than 150 yards, oor but near: yellow (default color)
		arrow:SetVertexColor(1,1,1)
	elseif distance<250000 then -- less than 500 yards, relatively far away: red
		arrow:SetVertexColor(1,0,0)
	else -- greater than 500 yards, very far away: brown
		arrow:SetVertexColor(.5,.5,1)
	end

	-- calculate angle between player facing and mouseover map position (trig courtesy of Javonthalas-Hyjal)
	local angle = atan2(unitY-playerY,unitX-playerX)*PI/180 - GetPlayerFacing()

	-- rotate arrow to the calculated angle; from http://wowpedia.org/SetTexCoord_Transformations
	local ULx,ULy = corner(angle+rads225)
	local LLx,LLy = corner(angle+rads135)
	local URx,URy = corner(angle-rads45)
	local LRx,LRy = corner(angle+rads45)
	arrow:SetTexCoord(ULx,ULy,LLx,LLy,URx,URy,LRx,LRy)

	-- position arrow beside cursor
	local cursorX, cursorY = GetCursorPosition()
	local scale = UIParent:GetEffectiveScale()
	self:SetPoint("CENTER",UIParent,"BOTTOMLEFT",cursorX/scale+self.offset,cursorY/scale-12)

	if settings.ShowYards then
		pointer.yards:SetText(format("%.0f yds",sqrt(distance)))
	end

	self:Show()
end

-- When unit frames are beyond mouseover range, UPDATE_MOUSEOVER_UNIT does not fire.
-- There's no event to indicate when the mouse moves over a unit frame all the time
-- and tooltip methods are unreliable in case an addon makes its own tooltip.
-- This means an OnUpate is going to have to periodically check if we're over a
-- frame with a unit attribute.  To minimize the processing the OnUpdate has to do:
-- The WorldFame's OnEnter/OnLeave will be securely hooked to hide/show the frame,
-- making the OnUpdate only run while the mouse is over a UI element. Also, there are
-- three speeds of polling: slow=mouse is not over a unit frame, mid=mouse is over
-- a unit frame but they're far away, fast=mouse is over a nearby unit.

pointer:SetScript("OnUpdate",function(self,elapsed)
	self.timer = self.timer - elapsed
	if self.timer < 0 then
		UpdateArrow(self.arrow)
	end
end)

WorldFrame:HookScript("OnEnter",function() pointer:Hide() end)
WorldFrame:HookScript("OnLeave",function() if IsInGroup() then pointer.timer=0 pointer:Show() end end)

--[[ No-Frills Interface Options Panel ]]

local opt = CreateFrame("Frame",nil,InterfaceOptionsFramePanelContainer)
opt:Hide()
opt.name = "M. OOR Pointer" -- note to self: don't make a ridiculously long addon name again

local title=opt:CreateFontString(nil,"ARTWORK","GameFontNormalLarge")
title:SetPoint("TOPLEFT",16,-16)
title:SetText("Mouseover Out of Range Pointer")
local ver=opt:CreateFontString(nil,"ARTWORK","GameFontNormalSmall")
ver:SetPoint("BOTTOMLEFT",title,"BOTTOMRIGHT",4,0)
ver:SetText("version "..GetAddOnMetadata("Mouseover Out of Range Pointer","Version"))
local desc=opt:CreateFontString(nil,"ARTWORK","GameFontHighlight")
desc:SetSize(InterfaceOptionsFramePanelContainer:GetWidth()-32,0)
desc:SetPoint("TOPLEFT",title,"BOTTOMLEFT",0,-8)
desc:SetJustifyH("LEFT")
desc:SetJustifyV("TOP")
desc:SetText("This addon displays an arrow beside your cursor to point towards the mouseover unit when they are out of range of spells.")

local optInfo = {
	{"ShowInRange","Show an arrow for mouseover units that are in range."},
	{"ShowYards","Show distance to mouseover unit in yards."},
	{"LargeArrow","Increase the size of the arrow by 50%."},
}

for i,info in ipairs(optInfo) do
	local check = CreateFrame("CheckButton",nil,opt,"UICheckButtonTemplate")
	check.var = info[1]
	check.text:SetFontObject("GameFontNormal")
	check.text:SetText(info[2])
	check:SetPoint("TOPLEFT",64,-100-(i-1)*32)
	check:SetScript("OnClick",function(self)
		MOORP_Settings[self.var] = self:GetChecked()
		if pointer.optfunc[self.var] then
			pointer.optfunc[self.var]()
		end
	end)
	info[3] = check
end

opt.refresh = function()
	for _,info in ipairs(optInfo) do
		info[3]:SetChecked(MOORP_Settings[info[1]] and true)
	end
end

InterfaceOptions_AddCategory(opt)

--[[ Option-related Extras ]]

pointer:SetScript("OnEvent",function(self)
	settings = MOORP_Settings
	for _,info in ipairs(optInfo) do
		if self.optfunc[info[1]] then
			self.optfunc[info[1]]()
		end
	end
end)
pointer:RegisterEvent("PLAYER_LOGIN")

pointer.optfunc = {}

function pointer.optfunc.LargeArrow()
	local size = settings.LargeArrow and 48 or 32
	pointer.arrow.offset = settings.LargeArrow and -20 or -10
	pointer.arrow:SetSize(size,size)
end

function pointer.optfunc.ShowYards()
	pointer.yards:SetShown(settings.ShowYards)
	pointer.yardsback:SetShown(settings.ShowYards)
end
